package be.jeffcheasey88.todo.model;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import dev.peerat.framework.utils.json.Json;
import dev.peerat.framework.utils.json.JsonArray;
import dev.peerat.framework.utils.json.JsonMap;
import dev.peerat.framework.utils.json.JsonParser;

public class JsonRepository implements Repository{
	
	private File dir;
	private JsonParser parser;
	
	private Function<String, String> tokenGenerator;
	
	public JsonRepository(File dir, Function<String, String> tokenGenerator){
		this.dir = dir.getAbsoluteFile();
		this.tokenGenerator = tokenGenerator;
		
		this.parser = new JsonParser();
	}
	
	private <J extends Json> J read(String path) throws Exception{
		File file = new File(dir, path).getAbsoluteFile();
		if(!file.getAbsolutePath().startsWith(dir.getAbsolutePath())) return null;
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String line;
		String content = "";
		while((line = reader.readLine()) != null) content+=line;
		reader.close();
		return parser.parse(content);
	}
	
	private boolean write(String path, Json json) throws Exception{
		File file = new File(dir, path).getAbsoluteFile();
		if(!file.getAbsolutePath().startsWith(dir.getAbsolutePath())) return false;
		BufferedWriter writer = new BufferedWriter(new FileWriter(file));
		writer.write(json.toString());
		writer.flush();
		writer.close();
		return true;
	}
	
	private <T> Result<T> error(int code, String message){
		return new Result<>(code, message);
	}

	@Override
	public Result<String> login(String username, String password){
		try {
			JsonMap map = read("users.json");
			if(map.has(username)){
				JsonMap infos = map.get(username);
				return password.equals(infos.get("password")) ? new Result<>(tokenGenerator.apply(username)) : new Result<>(401, "Wrong password");
			}
			return new Result<>(401,"Unknow username");
		}catch(Exception e){
			return error(500, "Internal Error");
		}
	}

	@Override
	public Result<List<String>> getProjects(UserWorker user){
		try {
			JsonMap map = read("users.json");
			List<String> result = new ArrayList<>();
			for(Object o : map.<JsonMap>get(user.getUsername()).<JsonArray>get("projects").toList()) result.add((String) o);
			return new Result<>(result);
		}catch(Exception e){
			return error(500, "Internal Error");
		}
	}

	@Override
	public Result<Project> createProject(UserWorker user, String name) {
		return null;
	}

	@Override
	public Result<Project> removeProject(UserWorker user, String name) {
		return null;
	}

	@Override
	public Result<Project> getProject(UserWorker user, String name) {
		return null;
	}

	@Override
	public Result<Boolean> renameProject(UserWorker user, String lastname, String updatedname) {
		return null;
	}

	@Override
	public Result<Boolean> manageUsers(UserWorker user, String project, List<String> users) {
		return null;
	}

	@Override
	public Result<List<String>> getUsers() {
		return null;
	}

	@Override
	public Result<List<String>> getUsers(String project) {
		return null;
	}

}
